@@ -48,22 +48,26 @@ enum {
   PROP_BACKEND,
   PROP_SCANNERS,
   PROP_SCANNER,
-  PROP_RESOLUTION,
+  PROP_XRESOLUTION,
+  PROP_YRESOLUTION,
   PROP_AREA,
   PROP_ROTATION
 };
 
 enum {
-  SIGNAL_PROBE_DONE,
-  SIGNAL_SCANNER_SELECTED,
-  SIGNAL_ACQUISITION_STARTED,
-  SIGNAL_DATA_RECEIVED,
-  SIGNAL_ACQUISITION_TERMINATED,
-  SIGNAL_IMAGE_RECEIVED,
-  SIGNAL_PREVIEW_STARTED,
-  SIGNAL_PREVIEW_TERMINATED,
-  SIGNAL_PREVIEW_RECEIVED,
-  SIGNAL_ERROR,
+  CHANGED,
+  PROBE_DONE,
+  SCANNER_SELECTED,
+  ACQUISITION_STARTED,
+  DATA_RECEIVED,
+  ACQUISITION_PAUSED,
+  ACQUISITION_RESUMED,
+  ACQUISITION_TERMINATED,
+  IMAGE_RECEIVED,
+  PREVIEW_STARTED,
+  PREVIEW_TERMINATED,
+  PREVIEW_RECEIVED,
+  ERROR,
   N_SIGNALS
 };
 
@@ -72,12 +76,16 @@ static guint signals[N_SIGNALS];
 void				gsc_trigger_error				(GnomeScanContext *context,
 										 GError *error);
 
-void				gsc_acquisition_threaded			(GnomeScanContext *context);
+void				gsc_acquire					(GnomeScanContext *context);
 
 void				gsc_data_received				(GnomeScanBackend *backend,
 										 gsize data_size,
 										 GnomeScanContext *context);
 
+void				gsc_image_acquired				(GnomeScanBackend *backend,
+										 GnomeScanResult *result,
+										 GnomeScanContext *context);
+
 /********************************
  * 	      GOBJECT		*
  ********************************/
@@ -138,10 +146,20 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
 							G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
-				   PROP_RESOLUTION,
-				   g_param_spec_int ("resolution",
-						     "Resolution",
-						     "The final image resolution.",
+				   PROP_XRESOLUTION,
+				   g_param_spec_int ("x-resolution",
+						     "X resolution",
+						     "The final image X resolution.",
+						     0,
+						     G_MAXINT,
+						     100,
+						     G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_YRESOLUTION,
+				   g_param_spec_int ("y-resolution",
+						     "Y resolution",
+						     "The final image y resolution.",
 						     0,
 						     G_MAXINT,
 						     100,
@@ -166,21 +184,39 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
 
   /* Signals */
 
+  /**
+   * GnomeScanContext::changed:
+   * @context:	The emitting #GnomeScanContext
+   * @property:	The changed property name
+   *
+   * Emitted when a scan option has been changed.
+   */
+  signals[CHANGED] = g_signal_new ("changed",
+				   GNOME_TYPE_SCAN_CONTEXT,
+				   G_SIGNAL_RUN_FIRST,
+				   G_STRUCT_OFFSET (GnomeScanContextClass, changed),
+				   NULL,
+				   NULL,
+				   g_cclosure_marshal_VOID__STRING,
+				   G_TYPE_NONE,
+				   1,
+				   G_TYPE_STRING);
+
   /**
    * GnomeScanContext::probe-done:
    * @context: The emitting #GnomeScanContext
    *
    * Emitted when the backend terminate the scanner probe.
    */
-  signals[SIGNAL_PROBE_DONE] = g_signal_new ("probe-done",
-					     GNOME_TYPE_SCAN_CONTEXT,
-					     G_SIGNAL_RUN_FIRST,
-					     G_STRUCT_OFFSET (GnomeScanContextClass, probe_done),
-					     NULL,
-					     NULL,
-					     g_cclosure_marshal_VOID__VOID,
-					     G_TYPE_NONE,
-					     0);
+  signals[PROBE_DONE] = g_signal_new ("probe-done",
+				      GNOME_TYPE_SCAN_CONTEXT,
+				      G_SIGNAL_RUN_FIRST,
+				      G_STRUCT_OFFSET (GnomeScanContextClass, probe_done),
+				      NULL,
+				      NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE,
+				      0);
 
   /**
    * GnomeScanContext::scanner-selected:
@@ -190,16 +226,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when the current a scanner is selected (wheter or not a
    * previous scanner were selected).
    */
-  signals[SIGNAL_SCANNER_SELECTED] = g_signal_new ("scanner-selected",
-						   GNOME_TYPE_SCAN_CONTEXT,
-						   G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-						   G_STRUCT_OFFSET (GnomeScanContextClass, scanner_selected),
-						   NULL,
-						   NULL,
-						   g_cclosure_marshal_VOID__OBJECT,
-						   G_TYPE_NONE,
-						   1,
-						   GNOME_TYPE_SCANNER);
+  signals[SCANNER_SELECTED] = g_signal_new ("scanner-selected",
+					    GNOME_TYPE_SCAN_CONTEXT,
+					    G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+					    G_STRUCT_OFFSET (GnomeScanContextClass, scanner_selected),
+					    NULL,
+					    NULL,
+					    g_cclosure_marshal_VOID__OBJECT,
+					    G_TYPE_NONE,
+					    1,
+					    GNOME_TYPE_SCANNER);
 
   /**
    * GnomeScanContext::preview-started:
@@ -209,16 +245,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when a preview acquisition is started.
    * 
    */
-  signals[SIGNAL_PREVIEW_STARTED] = g_signal_new ("preview-started",
-						  GNOME_TYPE_SCAN_CONTEXT,
-						  G_SIGNAL_RUN_FIRST,
-						  G_STRUCT_OFFSET (GnomeScanContextClass, preview_started),
-						  NULL,
-						  NULL,
-						  g_cclosure_marshal_VOID__BOXED,
-						  G_TYPE_NONE,
-						  1,
-						  GNOME_TYPE_SCAN_FORECAST);
+  signals[PREVIEW_STARTED] = g_signal_new ("preview-started",
+					   GNOME_TYPE_SCAN_CONTEXT,
+					   G_SIGNAL_RUN_FIRST,
+					   G_STRUCT_OFFSET (GnomeScanContextClass, preview_started),
+					   NULL,
+					   NULL,
+					   g_cclosure_marshal_VOID__BOXED,
+					   G_TYPE_NONE,
+					   1,
+					   GNOME_TYPE_SCAN_FORECAST);
 
   /**
    * GnomeScanContext::preview-terminated:
@@ -227,15 +263,15 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when a preview acquisition is terminated.
    * 
    */
-  signals[SIGNAL_PREVIEW_TERMINATED] = g_signal_new ("preview-terminated",
-						     GNOME_TYPE_SCAN_CONTEXT,
-						     G_SIGNAL_RUN_FIRST,
-						     G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
-						     NULL,
-						     NULL,
-						     g_cclosure_marshal_VOID__VOID,
-						     G_TYPE_NONE,
-						     0);
+  signals[PREVIEW_TERMINATED] = g_signal_new ("preview-terminated",
+					      GNOME_TYPE_SCAN_CONTEXT,
+					      G_SIGNAL_RUN_FIRST,
+					      G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
+					      NULL,
+					      NULL,
+					      g_cclosure_marshal_VOID__VOID,
+					      G_TYPE_NONE,
+					      0);
 
   /**
    * GnomeScanContext::preview-received:
@@ -245,16 +281,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when a preview image is received.
    * 
    */
-  signals[SIGNAL_PREVIEW_RECEIVED] = g_signal_new ("preview-received",
-						   GNOME_TYPE_SCAN_CONTEXT,
-						   G_SIGNAL_RUN_FIRST,
-						   G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
-						   NULL,
-						   NULL,
-						   g_cclosure_marshal_VOID__POINTER,
-						   G_TYPE_NONE,
-						   1,
-						   G_TYPE_POINTER);
+  signals[PREVIEW_RECEIVED] = g_signal_new ("preview-received",
+					    GNOME_TYPE_SCAN_CONTEXT,
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
+					    NULL,
+					    NULL,
+					    g_cclosure_marshal_VOID__BOXED,
+					    G_TYPE_NONE,
+					    1,
+					    GNOME_TYPE_SCAN_RESULT);
 
   /**
    * GnomeScanContext::acquisition-started:
@@ -264,35 +300,67 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when the acquisition is started. Use that signal to show
    * a popup before actualy receiving data.
    */
-  signals[SIGNAL_ACQUISITION_STARTED] = g_signal_new ("acquisition-started",
-						      GNOME_TYPE_SCAN_CONTEXT,
-						      G_SIGNAL_RUN_FIRST,
-						      G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_started),
-						      NULL,
-						      NULL,
-						      g_cclosure_marshal_VOID__BOXED,
-						      G_TYPE_NONE,
-						      1,
-						      GNOME_TYPE_SCAN_FORECAST);
+  signals[ACQUISITION_STARTED] = g_signal_new ("acquisition-started",
+					       GNOME_TYPE_SCAN_CONTEXT,
+					       G_SIGNAL_RUN_FIRST,
+					       G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_started),
+					       NULL,
+					       NULL,
+					       g_cclosure_marshal_VOID__BOXED,
+					       G_TYPE_NONE,
+					       1,
+					       GNOME_TYPE_SCAN_FORECAST);
 
   /**
-   * GnomeScanContext::data-received:2
+   * GnomeScanContext::data-received:
    * @context: The emitting #GnomeScanContext
    * @data_size: The current number of bytes received.
    *
    * Emitted when the backend send data. Either for a final picture
    * acquisition or for a preview refresh.
    */
-  signals[SIGNAL_DATA_RECEIVED] = g_signal_new ("data-received",
-						GNOME_TYPE_SCAN_CONTEXT,
-						G_SIGNAL_RUN_FIRST,
-						G_STRUCT_OFFSET (GnomeScanContextClass, data_received),
-						NULL,
-						NULL,
-						g_cclosure_marshal_VOID__INT,
-						G_TYPE_NONE,
-						1,
-						G_TYPE_INT);
+  signals[DATA_RECEIVED] = g_signal_new ("data-received",
+					 GNOME_TYPE_SCAN_CONTEXT,
+					 G_SIGNAL_RUN_FIRST,
+					 G_STRUCT_OFFSET (GnomeScanContextClass, data_received),
+					 NULL,
+					 NULL,
+					 g_cclosure_marshal_VOID__INT,
+					 G_TYPE_NONE,
+					 1,
+					 G_TYPE_INT);
+
+  /**
+   * GnomeScanContext::acquisition-paused:
+   * @context: The emitting #GnomeScanContext
+   *
+   * Emitted when the acquisition process has been paused.
+   */
+  signals[ACQUISITION_PAUSED] = g_signal_new ("acquisition-paused",
+					      GNOME_TYPE_SCAN_CONTEXT,
+					      G_SIGNAL_RUN_FIRST,
+					      G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_paused),
+					      NULL,
+					      NULL,
+					      g_cclosure_marshal_VOID__VOID,
+					      G_TYPE_NONE,
+					      0);
+
+  /**
+   * GnomeScanContext::acquisition-resumed:
+   * @context: The emitting #GnomeScanContext
+   *
+   * Emitted when the acquisition process has been paused.
+   */
+  signals[ACQUISITION_RESUMED] = g_signal_new ("acquisition-resumed",
+					       GNOME_TYPE_SCAN_CONTEXT,
+					       G_SIGNAL_RUN_FIRST,
+					       G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_resumed),
+					       NULL,
+					       NULL,
+					       g_cclosure_marshal_VOID__VOID,
+					       G_TYPE_NONE,
+					       0);
 
   /**
    * GnomeScanContext::acquisition-terminated:
@@ -301,15 +369,15 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when the backend has no more data to send. This signal is
    * emitted even if the acquisition has been canceled.
    */
-  signals[SIGNAL_ACQUISITION_TERMINATED] = g_signal_new ("acquisition-terminated",
-							 GNOME_TYPE_SCAN_CONTEXT,
-							 G_SIGNAL_RUN_FIRST,
-							 G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_terminated),
-							 NULL,
-							 NULL,
-							 g_cclosure_marshal_VOID__VOID,
-							 G_TYPE_NONE,
-							 0);
+  signals[ACQUISITION_TERMINATED] = g_signal_new ("acquisition-terminated",
+						  GNOME_TYPE_SCAN_CONTEXT,
+						  G_SIGNAL_RUN_FIRST,
+						  G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_terminated),
+						  NULL,
+						  NULL,
+						  g_cclosure_marshal_VOID__VOID,
+						  G_TYPE_NONE,
+						  0);
 
   /**
    * GnomeScanContext::image-received:
@@ -320,16 +388,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * sent with that signal.
    * 
    */
-  signals[SIGNAL_IMAGE_RECEIVED] = g_signal_new ("image-received",
-						 GNOME_TYPE_SCAN_CONTEXT,
-						 G_SIGNAL_RUN_FIRST,
-						 G_STRUCT_OFFSET (GnomeScanContextClass, image_received),
-						 NULL,
-						 NULL,
-						 g_cclosure_marshal_VOID__POINTER,
-						 G_TYPE_NONE,
-						 1,
-						 G_TYPE_POINTER);
+  signals[IMAGE_RECEIVED] = g_signal_new ("image-received",
+					  GNOME_TYPE_SCAN_CONTEXT,
+					  G_SIGNAL_RUN_FIRST,
+					  G_STRUCT_OFFSET (GnomeScanContextClass, image_received),
+					  NULL,
+					  NULL,
+					  g_cclosure_marshal_VOID__BOXED,
+					  G_TYPE_NONE,
+					  1,
+					  GNOME_TYPE_SCAN_RESULT);
 
   /**
    * GnomeScanContext::error:
@@ -339,16 +407,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * This signal allow the application to avert he user that a backend
    * error occured.
    */
-  signals[SIGNAL_ERROR] = g_signal_new ("error",
-					GNOME_TYPE_SCAN_CONTEXT,
-					G_SIGNAL_RUN_FIRST,
-					G_STRUCT_OFFSET (GnomeScanContextClass, error),
-					NULL,
-					NULL,
-					g_cclosure_marshal_VOID__POINTER,
-					G_TYPE_NONE,
-					1,
-					G_TYPE_POINTER);
+  signals[ERROR] = g_signal_new ("error",
+				 GNOME_TYPE_SCAN_CONTEXT,
+				 G_SIGNAL_RUN_FIRST,
+				 G_STRUCT_OFFSET (GnomeScanContextClass, error),
+				 NULL,
+				 NULL,
+				 g_cclosure_marshal_VOID__POINTER,
+				 G_TYPE_NONE,
+				 1,
+				 G_TYPE_POINTER);
 }
 
 void
@@ -414,8 +482,11 @@ gnome_scan_context_set_property (GObject *obj,
   case PROP_SCANNER:
     priv->info->scanner = g_value_get_pointer (value);
     break;
-  case PROP_RESOLUTION:
-    priv->info->resolution = g_value_get_double (value);
+  case PROP_XRESOLUTION:
+    priv->info->xresolution = g_value_get_double (value);
+    break;
+  case PROP_YRESOLUTION:
+    priv->info->yresolution = g_value_get_double (value);
     break;
   case PROP_AREA:
     priv->info->area = g_value_get_pointer (value);
@@ -444,8 +515,11 @@ gnome_scan_context_get_property (GObject *obj,
   case PROP_SCANNER:
     g_value_set_pointer (value, priv->info->scanner);
     break;
-  case PROP_RESOLUTION:
-    g_value_set_double (value, priv->info->resolution);
+  case PROP_XRESOLUTION:
+    g_value_set_double (value, priv->info->xresolution);
+    break;
+  case PROP_YRESOLUTION:
+    g_value_set_double (value, priv->info->yresolution);
     break;
   case PROP_AREA:
     g_value_set_pointer (value, priv->info->area);
@@ -475,6 +549,7 @@ gnome_scan_context_new (void)
 {
   GError *error = NULL;
   GnomeScanBackend *backend = gnome_scan_backend_new (&error);
+  GnomeScanContextInfo *info;
 
   if (error) {
     g_critical ("Failed to initialise GnomeScanBackend.");
@@ -485,14 +560,24 @@ gnome_scan_context_new (void)
 					    "backend", backend,
 					    NULL);
 
+  /* Init info */
+  info = GET_PRIVATE (context)->info;
+  info->depth = 8;
+  info->colorspace = GNOME_SCAN_COLORSPACE_RGB;
+
   /* Is here the right place to do that ? */
-  if (!g_thread_supported ()) g_thread_init (NULL);
+/*   if (!g_thread_supported ()) g_thread_init (NULL); */
 
   g_signal_connect (backend,
 		    "data-received",
 		    (GCallback) gsc_data_received,
 		    context);
 
+  g_signal_connect (backend,
+		    "image-acquired",
+		    (GCallback) gsc_image_acquired,
+		    context);
+
   return context;
 }
 
@@ -607,7 +692,7 @@ gnome_scan_context_select_scanner (GnomeScanContext *context,
 }
 
 /**
- * gnome_scan_context_set_resolution:
+ * gnome_scan_context_set_resolutions:
  * @context: a #GnomeScanContext
  * @resolution: the new resolution
  * 
@@ -615,28 +700,31 @@ gnome_scan_context_select_scanner (GnomeScanContext *context,
  * allowed value.
  */
 void
-gnome_scan_context_set_resolution (GnomeScanContext *context,
-				   gdouble resolution)
+gnome_scan_context_set_resolutions (GnomeScanContext *context,
+				   gdouble xresolution,
+				    gdouble yresolution)
 {
   g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
-
-  GET_PRIVATE (context)->info->resolution = resolution;
+  GET_PRIVATE (context)->info->xresolution = xresolution;
+  GET_PRIVATE (context)->info->yresolution = yresolution;
+  g_signal_emit_by_name (context, "changed", "x-resolution");
+  g_signal_emit_by_name (context, "changed", "y-resolution");
 }
 
 /**
- * gnome_scan_context_get_resolution:
+ * gnome_scan_context_get_resolutions:
  * @context: a #GnomeScanContext
  * 
  * Retreive the current resolution the image will be scan on.
- * 
- * Return value: The current resolution.
  */
-gdouble
-gnome_scan_context_get_resolution (GnomeScanContext *context)
+void
+gnome_scan_context_get_resolutions (GnomeScanContext *context,
+				   gdouble *xresolution,
+				   gdouble *yresolution)
 {
-  g_return_val_if_fail (GNOME_IS_SCAN_CONTEXT (context), 0);
-
-  return GET_PRIVATE (context)->info->resolution;
+  g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
+  *xresolution = GET_PRIVATE (context)->info->xresolution;
+  *yresolution = GET_PRIVATE (context)->info->yresolution;
 }
 
 /**
@@ -658,6 +746,7 @@ gnome_scan_context_set_area (GnomeScanContext *context,
   g_free (priv->info->area);
   priv->info->area = g_memdup (area, sizeof (GnomeScanArea));
   g_debug ("Area set to %f×%f+%f+%f (mm)", area->width, area->height, area->x, area->y);
+  g_signal_emit_by_name (context, "changed", "area");
 }
 
 /**
@@ -690,6 +779,7 @@ gnome_scan_context_set_rotation (GnomeScanContext *context,
 {
   g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
   GET_PRIVATE (context)->rotation = rotation;
+  g_signal_emit_by_name (context, "changed", "rotation");
 }
 
 /**
@@ -709,16 +799,17 @@ gnome_scan_context_get_rotation (GnomeScanContext *context)
 
 /**
  * gnome_scan_context_set_source:
- * @context: 
- * @source: 
+ * @context:	a #GonmeScanContext
+ * @source: 	the source to select
  * 
- * Retreive the current selected source.
+ * Change current selected source.
  **/
 void
 gnome_scan_context_set_source (GnomeScanContext *context,
 			       GnomeScannerSource source)
 {
   GET_PRIVATE (context)->info->source = source;
+  g_signal_emit_by_name (context, "changed", "source");
 }
 
 /**
@@ -735,6 +826,68 @@ gnome_scan_context_get_source (GnomeScanContext *context)
   return GET_PRIVATE (context)->info->source;
 }
 
+/**
+ * gnome_scan_context_get_colorspace:
+ * @context: a #GnomeScanContext
+ * 
+ * Get the colorspace
+ * 
+ * Returns: The selected colorspace
+ **/
+GnomeScanColorspace
+gnome_scan_context_get_colorspace (GnomeScanContext *context)
+{
+  return GET_PRIVATE (context)->info->colorspace;
+}
+
+/**
+ * gnome_scan_context_set_colorspace:
+ * @context:		a #GnomeScanContext 
+ * @colorspace: 	a #GnomeScanColorspace
+ * 
+ * Choose the colorspace of the final image.
+ **/
+void
+gnome_scan_context_set_colorspace (GnomeScanContext *context,
+				   GnomeScanColorspace colorspace)
+{
+  GET_PRIVATE (context)->info->colorspace = colorspace;
+  g_signal_emit_by_name (context, "changed", "colorspace");
+}
+
+/**
+ * gnome_scan_context_get_depth:
+ * @context:	a #GnomeScanContext 
+ * 
+ * Get the final image depth
+ * 
+ * Returns: an integer
+ **/
+gint
+gnome_scan_context_get_depth (GnomeScanContext *context)
+{
+  return GET_PRIVATE (context)->info->depth;
+}
+
+/**
+ * gnome_scan_context_set_depth:
+ * @context:	a #GnomeScanContext 
+ * @depth: 	a depth
+ * 
+ * Set the image depth (e.g. bit per sample) of the final
+ * image. Currently, only 8bit depth is allow due to GdkPixbuf
+ * Limitation.
+ **/
+void
+gnome_scan_context_set_depth (GnomeScanContext *context,
+			      gint depth)
+{
+  g_return_if_fail (depth == 8);
+  GET_PRIVATE (context)->info->depth = depth;
+  g_signal_emit_by_name (context, "changed", "depth");
+}
+
+
 
 /**
  * gnome_scan_context_acquire_preview
@@ -761,9 +914,11 @@ gnome_scan_context_acquire_preview (GnomeScanContext *context)
   priv = GET_PRIVATE (context);
   scanner = priv->info->scanner;
 
+  /* We create a temp ContextInfo suitable for preview. */
   /* resolution */
-  info = g_new0 (GnomeScanContextInfo, 1);
-  info->resolution = 50;
+  info = g_memdup (priv->info, sizeof (GnomeScanContextInfo));
+  info->xresolution = 50;
+  info->yresolution = 50;
   info->preview = TRUE;
 
   /* area */
@@ -771,29 +926,13 @@ gnome_scan_context_acquire_preview (GnomeScanContext *context)
   info->area = g_new0 (GnomeScanArea, 1);
   info->area->width = geometry->width;
   info->area->height = geometry->height;
-  info->scanner = priv->info->scanner;
-  info->source = priv->info->source;
 
   fc =  gnome_scan_backend_forecast (GET_PRIVATE (context)->backend, info, &error);
   if (error) gsc_trigger_error (context, error);
-  g_signal_emit_by_name (context, "preview-started", fc);
-  while (gtk_events_pending ())  gtk_main_iteration ();
 
-  result = gnome_scan_backend_acquire (priv->backend, info, &error);
-  g_signal_emit_by_name (context, "preview-terminated");
-  while (gtk_events_pending ())  gtk_main_iteration ();
-
-  if (error || !result->image) {
-    if (error) {
-      gsc_trigger_error (context, error);
-    }
-  }
-  else {
-    g_debug ("preview-received");
-    g_signal_emit_by_name (context, "preview-received", result);
-  }
-
-  g_boxed_free (GNOME_TYPE_SCAN_RESULT, result);
+  g_signal_emit_by_name (context, "preview-started", fc); while (gtk_events_pending ())  gtk_main_iteration ();
+  gnome_scan_backend_acquire (priv->backend, info, &error);
+  g_signal_emit_by_name (context, "preview-terminated"); while (gtk_events_pending ())  gtk_main_iteration ();
 }
 
 /**
@@ -823,55 +962,44 @@ gnome_scan_context_forecast (GnomeScanContext *context)
  * @context: a #GnomeScanContext
  * 
  * Acquire a picture. Cancel an acquisition is possible using
- * gnome_scan_context_cancel_acquisition(). The resulted pixbuf is
- * sent using the #GnomeScanContext::"image-received" signal. If the
- * acquisition has been cancel or something else went wrong, only the
- * #GnomeScancontext::"acquisition-terminated" is sent.
+ * gnome_scan_context_stop_acquisition(). The resulted pixbuf is
+ * sent using the #GnomeScanContext::"image-received" signal.
  */
 void
 gnome_scan_context_start_acquisition (GnomeScanContext *context)
 {
-  GnomeScanContextPrivate *priv = GET_PRIVATE (context);
-  GnomeScanResult *result;
-  GdkPixbuf *image;
-  GError *error = NULL;
-
   g_signal_emit_by_name (context, "acquisition-started", gnome_scan_context_forecast (context));
   while (gtk_events_pending ()) gtk_main_iteration ();
-  result = gnome_scan_backend_acquire (priv->backend, priv->info, &error);
-  g_signal_emit_by_name (context, "acquisition-terminated");
-  while (gtk_events_pending ())  gtk_main_iteration ();
-
-  /* If there ware no error (no cancel, etc.) we have a new image. sent it. */
-  if (error || !result->image) {
-    if (error) {
-      gsc_trigger_error (context, error);
-    }
-  }
-  else {
-    /* apply transformations */
-    if (priv->rotation) {
-      image = result->image;
-      result->image = gdk_pixbuf_rotate_simple (image, priv->rotation);
-      g_object_unref (image);
-    }
-
-    g_signal_emit_by_name (context, "image-received", result);
-  }
-  g_boxed_free (GNOME_TYPE_SCAN_RESULT, result);
+  gsc_acquire (context);
 }
 
 /**
- * gnome_scan_context_cancel_acquisition:
+ * gnome_scan_context_resume_acquisition:
+ * @context: a #GnomeScanContext
+ *
+ * Continue an acquisition for multipage document.
+ */
+void
+gnome_scan_context_resume_acquisition (GnomeScanContext *context)
+{
+  g_signal_emit_by_name (context, "acquisition-resumed");
+  gsc_acquire (context);
+}
+
+/**
+ * gnome_scan_context_stop_acquisition:
  * @context: a #GnomeScanContext
  * 
- * Cancel either preview or final acquisition.
+ * Stop an atomic multiple acquisition. If an image is being aquired,
+ * the image acquisition is canceled.
  **/
 void
-gnome_scan_context_cancel_acquisition (GnomeScanContext *context)
+gnome_scan_context_stop_acquisition (GnomeScanContext *context)
 {
   g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
-  gnome_scan_backend_cancel (GET_PRIVATE (context)->backend, GET_PRIVATE (context)->info->scanner);
+  gnome_scan_backend_stop (GET_PRIVATE (context)->backend, GET_PRIVATE (context)->info->scanner);
+  g_signal_emit_by_name (context, "acquisition-terminated");
+  while (gtk_events_pending ())  gtk_main_iteration ();
 }
 
 
@@ -890,6 +1018,15 @@ gsc_trigger_error (GnomeScanContext *context,
   g_signal_emit_by_name (context, "error", error);
 }
 
+void
+gsc_acquire (GnomeScanContext *context)
+{
+  GnomeScanContextPrivate *priv = GET_PRIVATE (context);
+  GError *error = NULL;
+  gnome_scan_backend_acquire (priv->backend, priv->info, &error);
+  g_signal_emit_by_name (context, "acquisition_paused");
+}
+
 /* This function just propagate data-received backend signal. */
 void
 gsc_data_received (GnomeScanBackend *backend,
@@ -901,3 +1038,27 @@ gsc_data_received (GnomeScanBackend *backend,
 
   g_signal_emit_by_name (context, "data-received", data_size);
 }
+
+void
+gsc_image_acquired (GnomeScanBackend *backend,
+		    GnomeScanResult *result,
+		    GnomeScanContext *context)
+{
+  GnomeScanContextPrivate *priv = GET_PRIVATE (context);
+  GdkPixbuf *image;
+
+  if (result->preview) {
+    g_signal_emit_by_name (context, "preview-received", result);
+  }
+  else {
+    /* apply transformations */
+    if (priv->rotation) {
+      result = g_boxed_copy (GNOME_TYPE_SCAN_RESULT, result);
+      image = result->image;
+      result->image = gdk_pixbuf_rotate_simple (image, priv->rotation);
+      g_debug ("%s: Unref image %p", __FUNCTION__, image);
+      g_object_unref (image);
+    }
+    g_signal_emit_by_name (context, "image-received", result);
+  }
+}
